@@ -1,7 +1,8 @@
 // Copyright (C) 2023 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include "qqmlgrpccalloptions_p.h"
+
 #include <chrono>
 
 QT_BEGIN_NAMESPACE
@@ -27,7 +28,7 @@ void QQmlGrpcCallOptions::setDeadline(qint64 value)
     emit deadlineChanged();
 }
 
-QGrpcCallOptions QQmlGrpcCallOptions::options() const
+const QGrpcCallOptions &QQmlGrpcCallOptions::options() const
 {
     return m_options;
 }
@@ -43,6 +44,8 @@ void QQmlGrpcCallOptions::setMetadata(QQmlGrpcMetadata *value)
         m_metadata = value;
         if (m_metadata)
             m_options.withMetadata(m_metadata->metadata());
+        else
+            m_options.withMetadata(QGrpcMetadata());
         emit metadataChanged();
     }
 }
