@@ -1,5 +1,5 @@
 // Copyright (C) 2023 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef QQMLGRPCCMETADATA_P_H
 #define QQMLGRPCCMETADATA_P_H
@@ -15,13 +15,13 @@
 // We mean it.
 //
 
-#include "qtgrpcquickexports.h"
+#include <QtGrpcQuick/qtgrpcquickexports.h>
 
-#include <QtCore/QMetaType>
-#include <QtCore/QVariant>
-#include <QtCore/QMap>
-#include <QtGrpc/qgrpcmetadata.h>
-#include <QtQml/qqml.h>
+#include <QtCore/qmap.h>
+#include <QtCore/qobject.h>
+#include <QtCore/qvariant.h>
+#include <QtGrpc/qgrpcdefs.h>
+#include <QtQml/qqmlregistration.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -31,14 +31,14 @@ class Q_GRPCQUICK_EXPORT QQmlGrpcMetadata : public QObject
     QML_NAMED_ELEMENT(GrpcMetadata)
     QML_ADDED_IN_VERSION(6, 7)
 
-    Q_PROPERTY(QVariantMap data READ data WRITE setData NOTIFY dataChanged)
+    Q_PROPERTY(QVariantMap data READ data WRITE setData NOTIFY dataChanged REQUIRED)
 public:
     QQmlGrpcMetadata(QObject *parent = nullptr);
-    ~QQmlGrpcMetadata();
+    ~QQmlGrpcMetadata() override;
 
     const QGrpcMetadata &metadata() const { return m_metadata; }
     const QVariantMap &data() const { return m_variantdata; }
-    void setData(const QVariantMap &value);
+    void setData(const QVariantMap &data);
 
 signals:
     void dataChanged();
