@@ -209,28 +209,29 @@ void MessageDeclarationPrinter::printProperties()
     for (int i = 0; i < numFields; ++i) {
         const FieldDescriptor *field = m_descriptor->field(i);
         const char *propertyTemplate = CommonTemplates::PropertyTemplate();
+        const auto propertyMap = common::producePropertyMap(field, m_descriptor);
         if (common::isOneofField(field)) {
-            const auto propertyMap = common::producePropertyMap(field, m_descriptor);
             m_printer->Print(propertyMap,
                              common::isPureMessage(field)
                                      ? CommonTemplates::PropertyOneofMessageTemplate()
                                      : CommonTemplates::PropertyOneofTemplate());
-            m_printer->Print(propertyMap, CommonTemplates::PropertyHasOneofTemplate());
+            m_printer->Print(propertyMap, CommonTemplates::PropertyHasFieldTemplate());
             continue;
         }
 
         if (common::isOptionalField(field)) {
-            const auto propertyMap = common::producePropertyMap(field, m_descriptor);
             m_printer->Print(propertyMap, CommonTemplates::PropertyOneofTemplate());
-            m_printer->Print(propertyMap, CommonTemplates::PropertyHasOneofTemplate());
+            m_printer->Print(propertyMap, CommonTemplates::PropertyHasFieldTemplate());
             continue;
         }
 
         if (common::isPureMessage(field)) {
-            propertyTemplate = CommonTemplates::PropertyMessageTemplate();
-        } else if (common::hasQmlAlias(field)) {
-            propertyTemplate = CommonTemplates::PropertyNonScriptableTemplate();
-        } else if (field->is_repeated() && !field->is_map()) {
+            m_printer->Print(propertyMap,  CommonTemplates::PropertyMessageTemplate());
+            m_printer->Print(propertyMap, CommonTemplates::PropertyHasFieldTemplate());
+            continue;
+        }
+
+        if (field->is_repeated() && !field->is_map()) {
             // Non-message list properties don't require an extra QQmlListProperty to access
             // their data, so the property name should not contain the 'Data' suffix
             if (field->type() == FieldDescriptor::TYPE_MESSAGE)
@@ -238,7 +239,7 @@ void MessageDeclarationPrinter::printProperties()
             else
                 propertyTemplate = CommonTemplates::PropertyRepeatedTemplate();
         }
-        m_printer->Print(common::producePropertyMap(field, m_descriptor), propertyTemplate);
+        m_printer->Print(propertyMap, propertyTemplate);
     }
 
     // Generate extra QML property, that can be used in QML context
@@ -250,16 +251,6 @@ void MessageDeclarationPrinter::printProperties()
                                  CommonTemplates::PropertyQmlMessageTemplate());
             }
         }
-
-
-        // Generate extra QmlListProperty that is mapped to list
-        for (int i = 0; i < numFields; ++i) {
-            const FieldDescriptor *field = m_descriptor->field(i);
-            if (common::hasQmlAlias(field)) {
-                m_printer->Print(common::producePropertyMap(field, m_descriptor),
-                                 CommonTemplates::PropertyNonScriptableAliasTemplate());
-            }
-        }
     }
 
     Outdent();
@@ -290,7 +281,9 @@ void MessageDeclarationPrinter::printGetters()
                     m_printer->Print(propertyMap,
                                      CommonTemplates::GetterMessageDeclarationTemplate());
                     m_printer->Print(propertyMap,
-                                     CommonTemplates::ClearMessageDeclarationTemplate());
+                                     Options::instance().hasQml()
+                                         ? CommonTemplates::ClearQmlMessageDeclarationTemplate()
+                                         : CommonTemplates::ClearMessageDeclarationTemplate());
                 } else {
                     m_printer->Print(propertyMap, CommonTemplates::GetterDeclarationTemplate());
                 }
@@ -321,7 +314,10 @@ void MessageDeclarationPrinter::printSetters()
                 if (common::isOptionalField(field)) {
                     m_printer->Print(propertyMap,
                                      CommonTemplates::SetterOneofDeclarationTemplate());
-                    m_printer->Print(propertyMap, CommonTemplates::ClearOneofDeclarationTemplate());
+                    m_printer->Print(propertyMap,
+                                     Options::instance().hasQml()
+                                         ? CommonTemplates::ClearQmlOneofDeclarationTemplate()
+                                         : CommonTemplates::ClearOneofDeclarationTemplate());
                     return;
                 }
 
@@ -345,10 +341,14 @@ void MessageDeclarationPrinter::printSetters()
                     break;
                 }
             });
-    common::iterateOneofFields(
-            m_descriptor, [&](const OneofDescriptor *, const PropertyMap &propertyMap) {
-                m_printer->Print(propertyMap, CommonTemplates::ClearOneofDeclarationTemplate());
-            });
+    common::
+        iterateOneofFields(m_descriptor,
+                           [&](const OneofDescriptor *, const PropertyMap &propertyMap) {
+                               m_printer->Print(propertyMap,
+                                           Options::instance().hasQml()
+                                               ? CommonTemplates::ClearQmlOneofDeclarationTemplate()
+                                               : CommonTemplates::ClearOneofDeclarationTemplate());
+                           });
     Outdent();
 }
 
@@ -370,8 +370,6 @@ void MessageDeclarationPrinter::printPrivateGetters()
                 } else if (common::isPureMessage(field)) {
                     m_printer->Print(propertyMap,
                                     CommonTemplates::PrivateGetterMessageDeclarationTemplate());
-                } else if (common::hasQmlAlias(field)) {
-                    m_printer->Print(propertyMap, CommonTemplates::GetterNonScriptableDeclarationTemplate());
                 }
             });
     Outdent();
@@ -395,8 +393,6 @@ void MessageDeclarationPrinter::printPrivateSetters()
                 } else if (common::isPureMessage(field)) {
                     m_printer->Print(propertyMap,
                                     CommonTemplates::PrivateSetterMessageDeclarationTemplate());
-                } else if (common::hasQmlAlias(field)) {
-                    m_printer->Print(propertyMap, CommonTemplates::SetterNonScriptableDeclarationTemplate());
                 }
             });
     Outdent();
@@ -421,7 +417,7 @@ void MessageDeclarationPrinter::printQEnums()
         Indent();
         for (int j = 0; j < enumDescr->value_count(); ++j) {
             const auto *valueDescr = enumDescr->value(j);
-            m_printer->Print({ { "enumvalue", valueDescr->name() },
+            m_printer->Print({ { "enumvalue", common::qualifiedCppName(valueDescr->name()) },
                                { "value", std::to_string(valueDescr->number()) } },
                              CommonTemplates::EnumFieldTemplate());
         }
