@@ -4,6 +4,7 @@
 #include <grpcclienttestbase.h>
 
 #include <QtGrpc/QGrpcCallOptions>
+#include <QtGrpc/QGrpcClientInterceptorManager>
 
 #include <QtCore/QCryptographicHash>
 #include <QtCore/QFile>
@@ -13,8 +14,9 @@
 #include <QtTest/QSignalSpy>
 #include <QtTest/QTest>
 
-#include <testservice_client.grpc.qpb.h>
+#include <interceptormocks.h>
 #include <message_latency_defs.h>
+#include <testservice_client.grpc.qpb.h>
 
 using namespace Qt::Literals::StringLiterals;
 using namespace qtgrpc::tests;
@@ -25,7 +27,7 @@ class QtGrpcClientServerStreamTest : public GrpcClientTestBase
 public:
     QtGrpcClientServerStreamTest()
         : GrpcClientTestBase(
-                { GrpcClientTestBase::Channel::Qt, GrpcClientTestBase::Channel::Native })
+                Channels{ GrpcClientTestBase::Channel::Qt })
     {
     }
 
@@ -41,6 +43,9 @@ private slots:
     void CancelWhileErrorTimeout();
     void Deadline_data();
     void Deadline();
+    void Interceptor();
+    void CancelledInterceptor();
+    void InterceptResponse();
 };
 
 void QtGrpcClientServerStreamTest::Valid()
@@ -59,9 +64,12 @@ void QtGrpcClientServerStreamTest::Valid()
     QVERIFY(streamErrorSpy.isValid());
 
     QSignalSpy streamFinishedSpy(stream.get(), &QGrpcServerStream::finished);
+    QVERIFY(streamFinishedSpy.isValid());
+
     QObject::connect(stream.get(), &QGrpcServerStream::messageReceived, this, [&result, stream] {
-        SimpleStringMessage ret = stream->read<SimpleStringMessage>();
-        result.setTestFieldString(result.testFieldString() + ret.testFieldString());
+        const auto ret = stream->read<SimpleStringMessage>();
+        QVERIFY(ret.has_value());
+        result.setTestFieldString(result.testFieldString() + ret->testFieldString());
     });
 
     QTRY_COMPARE_EQ_WITH_TIMEOUT(streamFinishedSpy.count(), 1,
@@ -89,8 +97,9 @@ void QtGrpcClientServerStreamTest::Cancel()
 
     int i = 0;
     QObject::connect(stream.get(), &QGrpcServerStream::messageReceived, this, [&] {
-        SimpleStringMessage ret = stream->read<SimpleStringMessage>();
-        result.setTestFieldString(result.testFieldString() + ret.testFieldString());
+        const auto ret = stream->read<SimpleStringMessage>();
+        QVERIFY(ret.has_value());
+        result.setTestFieldString(result.testFieldString() + ret->testFieldString());
         if (++i == ExpectedMessageCount)
             stream->cancel();
     });
@@ -123,8 +132,9 @@ void QtGrpcClientServerStreamTest::DeferredCancel()
 
     int i = 0;
     QObject::connect(stream.get(), &QGrpcServerStream::messageReceived, this, [&] {
-        SimpleStringMessage ret = stream->read<SimpleStringMessage>();
-        result.setTestFieldString(result.testFieldString() + ret.testFieldString());
+        const auto ret = stream->read<SimpleStringMessage>();
+        QVERIFY(ret.has_value());
+        result.setTestFieldString(result.testFieldString() + ret->testFieldString());
         if (++i == ExpectedMessageCount)
             QTimer::singleShot(MessageLatencyThreshold, stream.get(), &QGrpcServerStream::cancel);
     });
@@ -155,8 +165,9 @@ void QtGrpcClientServerStreamTest::HugeBlob()
     QVERIFY(streamErrorSpy.isValid());
 
     QObject::connect(stream.get(), &QGrpcServerStream::messageReceived, this, [&result, stream] {
-        BlobMessage ret = stream->read<BlobMessage>();
-        result.setTestBytes(ret.testBytes());
+        const auto ret = stream->read<BlobMessage>();
+        QVERIFY(ret.has_value());
+        result.setTestBytes(ret->testBytes());
     });
 
     QTRY_COMPARE_EQ_WITH_TIMEOUT(streamFinishedSpy.count(), 1, MessageLatencyWithThreshold);
@@ -193,7 +204,11 @@ void QtGrpcClientServerStreamTest::GetAsyncReply()
     request.setTestFieldString("Hello Qt!");
 
     reply = client()->testMethod(request);
-    reply->subscribe(this, [reply, &result] { result = reply->read<SimpleStringMessage>(); });
+    reply->subscribe(this, [reply, &result] {
+        const auto ret = reply->read<SimpleStringMessage>();
+        QVERIFY(ret.has_value());
+        result = *ret;
+    });
 
     QTRY_COMPARE_WITH_TIMEOUT(result.testFieldString(), request.testFieldString(),
                               MessageLatencyWithThreshold);
@@ -202,9 +217,15 @@ void QtGrpcClientServerStreamTest::GetAsyncReply()
     request.setTestFieldString("Hello Qt1!");
 
     reply = client()->testMethod(request);
-    reply->subscribe(
-            this, [reply, &result] { result = reply->read<SimpleStringMessage>(); },
-            [] { QVERIFY(false); });
+    reply->subscribe(this, [reply, &result] {
+            const auto ret = reply->read<SimpleStringMessage>();
+            QVERIFY(ret.has_value());
+            result = *ret;
+        },
+        [] {
+            QVERIFY(false);
+        }
+    );
 
     QTRY_COMPARE_WITH_TIMEOUT(result.testFieldString(), request.testFieldString(),
                               MessageLatencyWithThreshold);
@@ -231,8 +252,9 @@ void QtGrpcClientServerStreamTest::MultipleStreams()
     QVERIFY(steamMessageRecievedSpy.isValid());
 
     QObject::connect(stream.get(), &QGrpcServerStream::messageReceived, this, [&result, stream] {
-        SimpleStringMessage ret = stream->read<SimpleStringMessage>();
-        result.setTestFieldString(result.testFieldString() + ret.testFieldString());
+        const auto ret = stream->read<SimpleStringMessage>();
+        QVERIFY(ret.has_value());
+        result.setTestFieldString(result.testFieldString() + ret->testFieldString());
     });
 
     QTRY_COMPARE_EQ_WITH_TIMEOUT(streamFinishedSpy.count(), 1,
@@ -313,9 +335,10 @@ void QtGrpcClientServerStreamTest::InThread()
         auto stream = client()->streamTestMethodServerStream(request);
         QObject::connect(stream.get(), &QGrpcServerStream::messageReceived, &waiter,
                          [&result, &i, &waiter, stream] {
-                             SimpleStringMessage ret = stream->read<SimpleStringMessage>();
+                             const auto ret = stream->read<SimpleStringMessage>();
+                             QVERIFY(ret.has_value());
                              result.setTestFieldString(result.testFieldString()
-                                                       + ret.testFieldString());
+                                                       + ret->testFieldString());
                              if (++i == 4)
                                  waiter.quit();
                          });
@@ -391,8 +414,9 @@ void QtGrpcClientServerStreamTest::Deadline()
 
     SimpleStringMessage result;
     QObject::connect(stream.get(), &QGrpcServerStream::messageReceived, this, [&result, stream] {
-        SimpleStringMessage ret = stream->read<SimpleStringMessage>();
-        result.setTestFieldString(result.testFieldString() + ret.testFieldString());
+        const auto ret = stream->read<SimpleStringMessage>();
+        QVERIFY(ret.has_value());
+        result.setTestFieldString(result.testFieldString() + ret->testFieldString());
     });
 
     if (timeout.count() < MessageLatency * ExpectedMessageCount) {
@@ -413,6 +437,140 @@ void QtGrpcClientServerStreamTest::Deadline()
     }
 }
 
+void QtGrpcClientServerStreamTest::Interceptor()
+{
+    const int ExpectedMessageCount = 4;
+    constexpr QLatin1StringView interceptor1Id = "int"_L1;
+    constexpr QLatin1StringView interceptor2Id = "er"_L1;
+
+    auto modifyFunc = [](std::shared_ptr<QGrpcChannelOperation> operation, QLatin1StringView id) {
+        SimpleStringMessage oldArg;
+        if (!operation->serializer()->deserialize(&oldArg, operation->argument())) {
+            QFAIL("Deserialization of arg failed.");
+            return;
+        }
+        SimpleStringMessage newArg;
+        newArg.setTestFieldString(QString("%1%2").arg(oldArg.testFieldString()).arg(id));
+        auto rawMessage = operation->serializer()->serialize(&newArg);
+        operation->setArgument(rawMessage);
+    };
+    auto manager = QGrpcClientInterceptorManager();
+    manager.registerInterceptors({ std::make_shared<MockInterceptor>(interceptor1Id, modifyFunc),
+                                   std::make_shared<MockInterceptor>(interceptor2Id, modifyFunc) });
+    auto channel = client()->channel();
+    channel->addInterceptorManager(manager);
+    client()->attachChannel(channel);
+
+    SimpleStringMessage result;
+    SimpleStringMessage request;
+    auto stream = client()->streamTestMethodServerStream(request);
+
+    QSignalSpy messageReceivedSpy(stream.get(), &QGrpcServerStream::messageReceived);
+    QVERIFY(messageReceivedSpy.isValid());
+    QSignalSpy streamErrorSpy(stream.get(), &QGrpcServerStream::errorOccurred);
+    QVERIFY(streamErrorSpy.isValid());
+
+    QSignalSpy streamFinishedSpy(stream.get(), &QGrpcServerStream::finished);
+    QObject::connect(stream.get(), &QGrpcServerStream::messageReceived, this, [&result, stream] {
+        const auto ret = stream->read<SimpleStringMessage>();
+        QVERIFY(ret.has_value());
+        result.setTestFieldString(result.testFieldString() + ret->testFieldString());
+    });
+
+    QTRY_COMPARE_EQ_WITH_TIMEOUT(streamFinishedSpy.count(), 1,
+                                 MessageLatencyWithThreshold * ExpectedMessageCount);
+    QCOMPARE(streamErrorSpy.count(), 0);
+
+    QCOMPARE(messageReceivedSpy.count(), ExpectedMessageCount);
+    QCOMPARE_EQ(result.testFieldString(), "inter1inter2inter3inter4");
+}
+
+void QtGrpcClientServerStreamTest::CancelledInterceptor()
+{
+    constexpr QLatin1StringView interceptor1Id = "inter1"_L1;
+
+    auto manager = QGrpcClientInterceptorManager();
+    manager.registerInterceptors({ std::make_shared<NoContinuationInterceptor>(interceptor1Id) });
+    auto channel = client()->channel();
+    channel->addInterceptorManager(manager);
+    client()->attachChannel(channel);
+
+    SimpleStringMessage result;
+    result.setTestFieldString("Result not changed by echo");
+    SimpleStringMessage request;
+    request.setTestFieldString("Stream");
+
+    auto stream = client()->streamTestMethodServerStream(request);
+
+    QSignalSpy streamFinishedSpy(stream.get(), &QGrpcServerStream::finished);
+    QVERIFY(streamFinishedSpy.isValid());
+    QSignalSpy streamErrorSpy(stream.get(), &QGrpcServerStream::errorOccurred);
+    QVERIFY(streamErrorSpy.isValid());
+
+    QObject::connect(stream.get(), &QGrpcServerStream::messageReceived, this, [&] {
+        const auto ret = stream->read<SimpleStringMessage>();
+        result.setTestFieldString(result.testFieldString() + ret->testFieldString());
+    });
+
+    QTRY_COMPARE_EQ_WITH_TIMEOUT(streamErrorSpy.count(), 1, MessageLatencyWithThreshold);
+    QCOMPARE(streamFinishedSpy.count(), 0);
+    QCOMPARE_EQ(result.testFieldString(), "Result not changed by echo");
+}
+
+void QtGrpcClientServerStreamTest::InterceptResponse()
+{
+    const int ExpectedMessageCount = 4;
+
+    SimpleStringMessage serverResponse;
+    auto interceptFunc =
+        [this, &serverResponse](std::shared_ptr<QGrpcChannelOperation> operation,
+                                std::shared_ptr<QGrpcServerStream> stream,
+                                QGrpcInterceptorContinuation<QGrpcServerStream> &continuation,
+                                QLatin1StringView) {
+            QObject::connect(stream.get(), &QGrpcServerStream::messageReceived, this,
+                             [&serverResponse, stream] {
+                                 const auto mess = stream->read<SimpleStringMessage>();
+                                 serverResponse.setTestFieldString(serverResponse.testFieldString()
+                                                                   + mess->testFieldString());
+                             });
+            continuation(std::move(stream), operation);
+        };
+
+    auto manager = QGrpcClientInterceptorManager();
+    manager.registerInterceptor(std::make_shared<ServerStreamInterceptor>("inter1"_L1,
+                                                                          interceptFunc));
+    auto channel = client()->channel();
+    channel->addInterceptorManager(manager);
+    client()->attachChannel(channel);
+
+    SimpleStringMessage request;
+    request.setTestFieldString("Stream");
+
+    auto stream = client()->streamTestMethodServerStream(request);
+
+    QSignalSpy streamFinishedSpy(stream.get(), &QGrpcServerStream::finished);
+    QVERIFY(streamFinishedSpy.isValid());
+    QSignalSpy streamErrorSpy(stream.get(), &QGrpcServerStream::errorOccurred);
+    QVERIFY(streamErrorSpy.isValid());
+    QSignalSpy messageReceivedSpy(stream.get(), &QGrpcServerStream::messageReceived);
+    QVERIFY(messageReceivedSpy.isValid());
+
+    SimpleStringMessage result;
+    QObject::connect(stream.get(), &QGrpcServerStream::messageReceived, this, [&] {
+        const auto ret = stream->read<SimpleStringMessage>();
+        QVERIFY(ret.has_value());
+        result.setTestFieldString(result.testFieldString() + ret->testFieldString());
+    });
+
+    QTRY_COMPARE_EQ_WITH_TIMEOUT(streamFinishedSpy.count(), 1,
+                                 MessageLatencyWithThreshold * ExpectedMessageCount);
+    QCOMPARE(streamErrorSpy.count(), 0);
+    QCOMPARE(messageReceivedSpy.count(), ExpectedMessageCount);
+
+    QCOMPARE_EQ(result.testFieldString(), "Stream1Stream2Stream3Stream4");
+    QCOMPARE_EQ(serverResponse.testFieldString(), "Stream1Stream2Stream3Stream4");
+}
+
 QTEST_MAIN(QtGrpcClientServerStreamTest)
 
 #include "tst_grpc_client_serverstream.moc"
