@@ -8,6 +8,7 @@
 #include <QtCore/QTimer>
 
 #include <QtGrpc/QGrpcCallOptions>
+#include <QtGrpc/QGrpcClientInterceptorManager>
 #include <QtGrpc/QGrpcMetadata>
 
 #include <QtTest/QSignalSpy>
@@ -15,13 +16,10 @@
 
 #include <memory>
 
-#if QT_CONFIG(native_grpc)
-#  include <grpcpp/security/credentials.h>
-#endif
-
-#include <testservice_client.grpc.qpb.h>
+#include <interceptormocks.h>
 #include <message_latency_defs.h>
 #include <server_proc_runner.h>
+#include <testservice_client.grpc.qpb.h>
 
 using namespace Qt::Literals::StringLiterals;
 using namespace qtgrpc::tests;
@@ -31,79 +29,78 @@ class QtGrpcClientUnaryCallTest : public GrpcClientTestBase
     Q_OBJECT
 public:
     QtGrpcClientUnaryCallTest()
-        : GrpcClientTestBase(
-                { GrpcClientTestBase::Channel::Qt, GrpcClientTestBase::Channel::Native })
+        : GrpcClientTestBase(GrpcClientTestBase::Channels{
+            GrpcClientTestBase::Channel::Qt,
+            GrpcClientTestBase::Channel::Json,
+            #if !defined(Q_OS_DARWIN) && !defined(Q_OS_WIN32)
+                            GrpcClientTestBase::Channel::Ssl,
+            #endif
+        })
     {
     }
 
 private slots:
-    void Blocking();
     void AsyncWithSubscribe();
     void AsyncWithLambda();
     void ImmediateCancel();
     void DeferredCancel();
     void AsyncClientStatusMessage();
-    void BlockingClientStatusMessage();
     void AsyncStatusMessage();
-    void BlockingStatusMessage();
-    void NonCompatibleArgRet();
     void InThread();
     void AsyncInThread();
     void Metadata();
     void Deadline_data();
     void Deadline();
+    void Interceptor();
+    void CancelledInterceptor();
+    void InterceptResponse();
+    void CacheIntercept();
 };
 
-void QtGrpcClientUnaryCallTest::Blocking()
-{
-    SimpleStringMessage request;
-    auto result = std::make_shared<SimpleStringMessage>();
-    request.setTestFieldString("Hello Qt!");
-    QCOMPARE_EQ(client()->testMethod(request, result.get()), QGrpcStatus::Ok);
-    QCOMPARE_EQ(result->testFieldString(), "Hello Qt!");
-}
-
 void QtGrpcClientUnaryCallTest::AsyncWithSubscribe()
 {
     SimpleStringMessage request;
-    SimpleStringMessage result;
     request.setTestFieldString("Hello Qt!");
-    QEventLoop waiter;
+    std::optional<SimpleStringMessage> result;
 
+    bool waitForReply = false;
     std::shared_ptr<QGrpcCallReply> reply = client()->testMethod(request);
-    reply->subscribe(this, [reply, &result, &waiter] {
+    reply->subscribe(this, [reply, &result, &waitForReply] {
         result = reply->read<SimpleStringMessage>();
-        waiter.quit();
+        waitForReply = true;
     });
 
-    waiter.exec();
-    QCOMPARE_EQ(result.testFieldString(), "Hello Qt!");
+    QTRY_COMPARE_EQ_WITH_TIMEOUT(waitForReply, true, MessageLatency);
+    QVERIFY(result.has_value());
+    QCOMPARE_EQ(result->testFieldString(), "Hello Qt!");
 }
 
 void QtGrpcClientUnaryCallTest::AsyncWithLambda()
 {
-    SimpleStringMessage result;
+    std::optional<SimpleStringMessage> result = SimpleStringMessage();
     SimpleStringMessage request;
     request.setTestFieldString("Hello Qt!");
-    QEventLoop waiter;
-    client()->testMethod(request, this, [&result, &waiter](std::shared_ptr<QGrpcCallReply> reply) {
-        result = reply->read<SimpleStringMessage>();
-        waiter.quit();
-    });
+    bool waitForReply = false;
+    client()->testMethod(request, this,
+                         [&result, &waitForReply](std::shared_ptr<QGrpcCallReply> reply) {
+                             result = reply->read<SimpleStringMessage>();
+                             waitForReply = true;
+                         });
 
-    waiter.exec();
-    QCOMPARE_EQ(result.testFieldString(), "Hello Qt!");
+    QTRY_COMPARE_EQ_WITH_TIMEOUT(waitForReply, true, MessageLatency);
+    QVERIFY(result.has_value());
+    QCOMPARE_EQ(result->testFieldString(), "Hello Qt!");
 }
 
 void QtGrpcClientUnaryCallTest::ImmediateCancel()
 {
-    SimpleStringMessage result;
     SimpleStringMessage request;
     request.setTestFieldString("sleep");
 
     std::shared_ptr<QGrpcCallReply> reply = client()->testMethod(request);
 
-    result.setTestFieldString("Result not changed by echo");
+    std::optional<SimpleStringMessage> result = SimpleStringMessage();
+    result->setTestFieldString("Result not changed by echo");
     QObject::connect(reply.get(), &QGrpcCallReply::finished, this,
                      [&result, reply] { result = reply->read<SimpleStringMessage>(); });
 
@@ -121,7 +118,9 @@ void QtGrpcClientUnaryCallTest::ImmediateCancel()
     QTRY_COMPARE_EQ_WITH_TIMEOUT(clientErrorSpy.count(), 1, FailTimeout);
     QTRY_COMPARE_EQ_WITH_TIMEOUT(replyFinishedSpy.count(), 0, FailTimeout);
 
-    QCOMPARE_EQ(result.testFieldString(), "Result not changed by echo");
+    QCOMPARE_EQ(result->testFieldString(), "Result not changed by echo");
+    QCOMPARE_EQ(qvariant_cast<QGrpcStatus>(clientErrorSpy.at(0).first()).code(),
+                QGrpcStatus::Cancelled);
 }
 
 void QtGrpcClientUnaryCallTest::DeferredCancel()
@@ -129,12 +128,14 @@ void QtGrpcClientUnaryCallTest::DeferredCancel()
     SimpleStringMessage request;
     request.setTestFieldString("sleep");
 
-    SimpleStringMessage result;
-    result.setTestFieldString("Result not changed by echo");
+    std::optional<SimpleStringMessage> result = SimpleStringMessage();
+    result->setTestFieldString("Result not changed by echo");
     std::shared_ptr<QGrpcCallReply> reply = client()->testMethod(request);
 
-    QObject::connect(reply.get(), &QGrpcCallReply::finished, this,
-                     [reply, &result] { result = reply->read<SimpleStringMessage>(); });
+    QObject::connect(reply.get(), &QGrpcCallReply::finished, this, [reply, &result] {
+        QVERIFY(false);
+        result = reply->read<SimpleStringMessage>();
+    });
 
     QSignalSpy replyErrorSpy(reply.get(), &QGrpcCallReply::errorOccurred);
     QVERIFY(replyErrorSpy.isValid());
@@ -142,7 +143,7 @@ void QtGrpcClientUnaryCallTest::DeferredCancel()
     QTimer::singleShot(MessageLatencyThreshold, reply.get(), &QGrpcCallReply::cancel);
 
     QTRY_COMPARE_EQ_WITH_TIMEOUT(replyErrorSpy.count(), 1, FailTimeout);
-    QCOMPARE_EQ(result.testFieldString(), "Result not changed by echo");
+    QCOMPARE_EQ(result->testFieldString(), "Result not changed by echo");
 }
 
 void QtGrpcClientUnaryCallTest::AsyncClientStatusMessage()
@@ -161,23 +162,6 @@ void QtGrpcClientUnaryCallTest::AsyncClientStatusMessage()
              request.testFieldString());
 }
 
-void QtGrpcClientUnaryCallTest::BlockingClientStatusMessage()
-{
-    SimpleStringMessage request;
-    request.setTestFieldString("Some status message");
-    auto result = std::make_shared<SimpleStringMessage>();
-
-    QSignalSpy clientErrorSpy(client().get(), &TestService::Client::errorOccurred);
-    QVERIFY(clientErrorSpy.isValid());
-
-    client()->testMethodStatusMessage(request, result.get());
-
-    QTRY_COMPARE_GE_WITH_TIMEOUT(clientErrorSpy.count(), 1, FailTimeout);
-
-    QCOMPARE(qvariant_cast<QGrpcStatus>(clientErrorSpy.at(0).first()).message(),
-             request.testFieldString());
-}
-
 void QtGrpcClientUnaryCallTest::AsyncStatusMessage()
 {
     SimpleStringMessage request;
@@ -193,49 +177,23 @@ void QtGrpcClientUnaryCallTest::AsyncStatusMessage()
              request.testFieldString());
 }
 
-
-void QtGrpcClientUnaryCallTest::BlockingStatusMessage()
-{
-    SimpleStringMessage request;
-    request.setTestFieldString("Some status message");
-    auto result = std::make_shared<SimpleStringMessage>();
-
-    QGrpcStatus status = client()->testMethodStatusMessage(request, result.get());
-
-    QCOMPARE_EQ(status.message(), request.testFieldString());
-}
-
-void QtGrpcClientUnaryCallTest::NonCompatibleArgRet()
-{
-    const QtProtobuf::sint32 TestValue = 2048;
-    const QString TestValueString = QString::number(TestValue);
-
-    SimpleIntMessage request;
-    request.setTestField(TestValue);
-    auto result = std::make_shared<SimpleStringMessage>();
-    QCOMPARE_EQ(client()->testMethodNonCompatibleArgRet(request, result.get()), QGrpcStatus::Ok);
-    QCOMPARE_EQ(result->testFieldString(), TestValueString);
-}
-
 void QtGrpcClientUnaryCallTest::InThread()
 {
     SimpleStringMessage request;
-    auto result = std::make_shared<SimpleStringMessage>();
 
     request.setTestFieldString("Hello Qt from thread!");
 
     QSignalSpy clientErrorSpy(client().get(), &TestService::Client::errorOccurred);
     QVERIFY(clientErrorSpy.isValid());
 
-    bool ok = false;
+    std::shared_ptr<QGrpcCallReply> reply;
     const std::unique_ptr<QThread> thread(QThread::create(
-            [&] { ok = client()->testMethod(request, result.get()) == QGrpcStatus::Ok; }));
+        [&] { reply = client()->testMethod(request); }));
 
     thread->start();
 
     QTRY_COMPARE_EQ_WITH_TIMEOUT(clientErrorSpy.count(), 1, MessageLatencyWithThreshold);
-    QVERIFY(!ok);
-    QVERIFY(result->testFieldString().isEmpty());
+    QVERIFY(reply == nullptr);
     QVERIFY(qvariant_cast<QGrpcStatus>(clientErrorSpy.at(0).first())
                     .message()
                     .startsWith("QAbstractGrpcClient::call is called from a different thread."));
@@ -244,12 +202,12 @@ void QtGrpcClientUnaryCallTest::InThread()
 void QtGrpcClientUnaryCallTest::AsyncInThread()
 {
     SimpleStringMessage request;
-    SimpleStringMessage result;
     request.setTestFieldString("Hello Qt from thread!");
 
     QSignalSpy clientErrorSpy(client().get(), &TestService::Client::errorOccurred);
     QVERIFY(clientErrorSpy.isValid());
 
+    std::optional<SimpleStringMessage> result = SimpleStringMessage();
     const std::unique_ptr<QThread> thread(QThread::create([&] {
         QEventLoop waiter;
         std::shared_ptr<QGrpcCallReply> reply = client()->testMethod(request);
@@ -263,7 +221,7 @@ void QtGrpcClientUnaryCallTest::AsyncInThread()
 
     thread->start();
     QTRY_COMPARE_EQ_WITH_TIMEOUT(clientErrorSpy.count(), 1, FailTimeout);
-    QTRY_VERIFY(result.testFieldString().isEmpty());
+    QTRY_VERIFY(result.has_value());
     QTRY_VERIFY(
             qvariant_cast<QGrpcStatus>(clientErrorSpy.at(0).first())
                     .message()
@@ -299,8 +257,6 @@ void QtGrpcClientUnaryCallTest::Metadata()
     }
 
     QCOMPARE_EQ(clientErrorSpy.count(), 0);
-    if (channelType().testFlag(GrpcClientTestBase::Channel::Native))
-        QEXPECT_FAIL("", "Unimplemented in the reference gRPC channel", Abort);
     QCOMPARE_EQ(serverHeaderCount, 1);
     QCOMPARE_EQ(clientReturnHeader, "valid_value"_ba);
 }
@@ -339,7 +295,7 @@ void QtGrpcClientUnaryCallTest::Deadline()
                 || code == QGrpcStatus::StatusCode::Unavailable);
     } else if (timeout.count() >= MessageLatencyWithThreshold) {
         QTRY_COMPARE_EQ_WITH_TIMEOUT(callFinishedSpy.count(), 1, MessageLatencyWithThreshold);
-        QCOMPARE(reply->read<SimpleStringMessage>().testFieldString(), request.testFieldString());
+        QCOMPARE(reply->read<SimpleStringMessage>()->testFieldString(), request.testFieldString());
     } else {
         // Because we're can't be sure about the result,
         // cancel the call, that might affect other tests.
@@ -347,6 +303,146 @@ void QtGrpcClientUnaryCallTest::Deadline()
     }
 }
 
+void QtGrpcClientUnaryCallTest::Interceptor()
+{
+    constexpr QLatin1StringView clientMetadataKey = "client_header"_L1;
+    constexpr QLatin1StringView serverMetadataKey = "server_header"_L1;
+    constexpr QLatin1StringView interceptor1Id = "inter1"_L1;
+    constexpr QLatin1StringView interceptor2Id = "inter2"_L1;
+
+    auto modifyFunc = [clientMetadataKey](std::shared_ptr<QGrpcChannelOperation> operation,
+                                          QLatin1StringView id) {
+        auto metadata = operation->options().metadata();
+        if (auto it = metadata.find(clientMetadataKey.latin1()); it != metadata.end()) {
+            it->second += id;
+        } else {
+            metadata.insert({ clientMetadataKey.latin1(), id.latin1() });
+        }
+        operation->setClientMetadata(metadata);
+    };
+    auto manager = QGrpcClientInterceptorManager();
+    manager.registerInterceptors({ std::make_shared<MockInterceptor>(interceptor1Id, modifyFunc),
+                                   std::make_shared<MockInterceptor>(interceptor2Id, modifyFunc) });
+    auto channel = client()->channel();
+    channel->addInterceptorManager(manager);
+    client()->attachChannel(channel);
+
+    auto reply = client()->testMetadata({});
+
+    QSignalSpy callFinishedSpy(reply.get(), &QGrpcCallReply::finished);
+    QVERIFY(callFinishedSpy.isValid());
+
+    QTRY_COMPARE_EQ_WITH_TIMEOUT(callFinishedSpy.count(), 1, MessageLatencyWithThreshold);
+    const auto serverMetadata = reply->metadata();
+    const auto it = serverMetadata.find(serverMetadataKey.latin1());
+
+    QVERIFY(it != serverMetadata.end());
+    QCOMPARE_EQ(it->second, "inter1inter2"_ba);
+}
+
+void QtGrpcClientUnaryCallTest::CancelledInterceptor()
+{
+    constexpr QLatin1StringView interceptor1Id = "inter1"_L1;
+
+    auto manager = QGrpcClientInterceptorManager();
+    manager.registerInterceptors({ std::make_shared<NoContinuationInterceptor>(interceptor1Id) });
+    auto channel = client()->channel();
+    channel->addInterceptorManager(manager);
+    client()->attachChannel(channel);
+
+    SimpleStringMessage request;
+    request.setTestFieldString("sleep");
+
+    std::shared_ptr<QGrpcCallReply> reply = client()->testMethod(request);
+
+    std::optional<SimpleStringMessage> result = SimpleStringMessage();
+    result->setTestFieldString("Result not changed by echo");
+    QObject::connect(reply.get(), &QGrpcCallReply::finished, this,
+                     [&result, reply] { result = reply->read<SimpleStringMessage>(); });
+
+    QSignalSpy replyErrorSpy(reply.get(), &QGrpcCallReply::errorOccurred);
+    QSignalSpy replyFinishedSpy(reply.get(), &QGrpcCallReply::finished);
+    QSignalSpy clientErrorSpy(client().get(), &TestService::Client::errorOccurred);
+
+    QVERIFY(replyErrorSpy.isValid());
+    QVERIFY(replyFinishedSpy.isValid());
+    QVERIFY(clientErrorSpy.isValid());
+
+    QTRY_COMPARE_EQ_WITH_TIMEOUT(replyErrorSpy.count(), 1, FailTimeout);
+    QTRY_COMPARE_EQ_WITH_TIMEOUT(clientErrorSpy.count(), 1, FailTimeout);
+    QTRY_COMPARE_EQ_WITH_TIMEOUT(replyFinishedSpy.count(), 0, FailTimeout);
+
+    QCOMPARE_EQ(result->testFieldString(), "Result not changed by echo");
+}
+
+void QtGrpcClientUnaryCallTest::InterceptResponse()
+{
+    std::optional<SimpleStringMessage> serverResponse = SimpleStringMessage();
+    auto interceptFunc =
+        [this, &serverResponse](std::shared_ptr<QGrpcChannelOperation> operation,
+                                std::shared_ptr<QGrpcCallReply> response,
+                                QGrpcInterceptorContinuation<QGrpcCallReply> &continuation,
+                                QLatin1StringView) {
+            QObject::connect(response.get(), &QGrpcCallReply::finished, this,
+                             [&serverResponse, response] {
+                                 serverResponse = response->read<SimpleStringMessage>();
+                             });
+            continuation(std::move(response), std::move(operation));
+        };
+
+    auto manager = QGrpcClientInterceptorManager();
+    manager.registerInterceptor(std::make_shared<CallInterceptor>("inter1"_L1, interceptFunc));
+    auto channel = client()->channel();
+    channel->addInterceptorManager(manager);
+    client()->attachChannel(channel);
+
+    SimpleStringMessage request;
+    request.setTestFieldString("Hello Qt!");
+    std::optional<SimpleStringMessage> result;
+    client()->testMethod(request, client().get(), [&result](std::shared_ptr<QGrpcCallReply> reply) {
+        result = reply->read<SimpleStringMessage>();
+    });
+
+    QTRY_COMPARE_EQ_WITH_TIMEOUT(serverResponse->testFieldString(),
+                                 "Hello Qt!", MessageLatencyWithThreshold);
+    QCOMPARE_EQ(result->testFieldString(), "Hello Qt!");
+}
+
+void QtGrpcClientUnaryCallTest::CacheIntercept()
+{
+    SimpleStringMessage serverResponse;
+    auto interceptFunc =
+        [](std::shared_ptr<QGrpcChannelOperation> operation, std::shared_ptr<QGrpcCallReply>,
+           QGrpcInterceptorContinuation<QGrpcCallReply> &, QLatin1StringView id) {
+            SimpleStringMessage deserializedArg;
+            if (!operation->serializer()->deserialize(&deserializedArg, operation->argument())) {
+                QFAIL("Deserialization of arg failed.");
+                return;
+            }
+            SimpleStringMessage cachedValue;
+            cachedValue.setTestFieldString(id);
+            const auto serializedValue = operation->serializer()->serialize(&cachedValue);
+            emit operation->dataReady(serializedValue);
+            emit operation->finished();
+        };
+
+    auto manager = QGrpcClientInterceptorManager();
+    manager.registerInterceptor(std::make_shared<CallInterceptor>("inter1"_L1, interceptFunc));
+    auto channel = client()->channel();
+    channel->addInterceptorManager(manager);
+    client()->attachChannel(channel);
+
+    SimpleStringMessage request;
+    std::optional<SimpleStringMessage> result = SimpleStringMessage();
+    request.setTestFieldString("Hello Qt!");
+    client()->testMethod(request, client().get(), [&result](std::shared_ptr<QGrpcCallReply> reply) {
+        result = reply->read<SimpleStringMessage>();
+    });
+
+    QTRY_COMPARE_EQ_WITH_TIMEOUT(result->testFieldString(),
+                                 "inter1", MessageLatencyWithThreshold);
+}
+
 QTEST_MAIN(QtGrpcClientUnaryCallTest)
 
 #include "tst_grpc_client_unarycall.moc"
