@@ -1,4 +1,4 @@
-#include <QtProtobuf/qprotobufserializer.h>
+#include <QtProtobuf/qprotobufregistration.h>
 #include "enummessages.qpb.h"
 
 namespace qtprotobufnamespace::tests {
@@ -17,5 +17,6 @@ static QtProtobuf::ProtoTypeRegistrar ProtoTypeRegistrarSimpleFileEnumMessage(qR
 static QtProtobuf::ProtoTypeRegistrar ProtoTypeRegistrarStepChildEnumMessage(qRegisterProtobufType<StepChildEnumMessage>);
 static QtProtobuf::ProtoTypeRegistrar ProtoTypeRegistrarA(qRegisterProtobufType<A>);
 static QtProtobuf::ProtoTypeRegistrar ProtoTypeRegistrarB(qRegisterProtobufType<B>);
+static bool RegisterEnummessagesProtobufTypes = [](){ qRegisterProtobufTypes(); return true; }();
 } // namespace qtprotobufnamespace::tests
 
